class CarBookingCurrentlyActiveException(BaseException):
    pass

class AlreadyBookedFlexError(BaseException):
    pass

class NoCurrentlyBookedFlex(BaseException):
    pass

class OverlappingWithBookingsWindow(BaseException):
    pass

class ExpiredSearchException(BaseException):
    pass

class AlreadyBookedFlexError(BaseException):
    pass

class NoCurrentlyBookedFlex(BaseException):
    pass

class CarBookingCurrentlyActiveException(BaseException):
    pass

class GetAuthTokenFailure(BaseException):
    pass

class RefreshAuthTokenFailure(BaseException):
    pass

class AuthenticatedClientRequiredException(BaseException):
    pass

class NoStationsWhithinRadiusException(BaseException):
    pass
